/*
 * menusched.c: EPG2VDR plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 */

#define __STL_CONFIG_H

#include <vdr/menuitems.h>
#include <vdr/status.h>
#include <vdr/menu.h>

#include "plgconfig.h"
#include "menu.h"

#include "ttools.h"

#if defined (APIVERSNUM) && (APIVERSNUM >= 20301)
#  warning "Program menu have to be ported to VDR >= 2.3.1, compiling without it"
#else

#define NEWTIMERLIMIT    60 // seconds until the start time of a new timer created from the Schedule menu,
                            // within which it will go directly into the "Edit timer" menu to allow
                            // further parameter settings

#define MAXCHNAMWIDTH    16 // maximum number of characters of channels' short names shown in schedules menus

#define CHNUMWIDTH       (numdigits(Channels.MaxNumber()) + 1)
#define CHNAMWIDTH       (std::min(MAXCHNAMWIDTH, Channels.MaxShortChannelNameLength() + 1))

//***************************************************************************
// Class cMenuEpgMatchRecordings
//***************************************************************************

cMenuEpgMatchRecordings::cMenuEpgMatchRecordings(cMenuDb* db, const cEvent* event)
   : cOsdMenu(tr("Matching recordings"), 30, 30, 30, 40)
{
   cMenuDb* menuDb = db;
   
   Add(new cOsdItem(tr("Direct Matches:")));
   cList<cOsdItem>::Last()->SetSelectable(no);
  
   tell(0, "Search recordings for '%s' '%s'", event->Title(), event->ShortText());

   menuDb->recordingListDb->clear();
   menuDb->recordingListDb->setValue("TITLE", event->Title());
   menuDb->recordingListDb->setValue("SHORTTEXT", event->ShortText());
   
   for (int f = menuDb->selectRecordingForEvent->find(); f; f = menuDb->selectRecordingForEvent->fetch())
   {
      const char* vdr = 0;
      
#ifdef WITH_PIN     
      if (!cOsd::pinValid && menuDb->recordingListDb->getIntValue("FSK"))
         continue;
#endif
      
      if (!menuDb->recordingListDb->getValue("OWNER")->isEmpty())
      {
         menuDb->vdrDb->clear();
         menuDb->vdrDb->setValue("UUID", menuDb->recordingListDb->getStrValue("OWNER"));
         
         if (menuDb->vdrDb->find())
            vdr = menuDb->vdrDb->getStrValue("NAME");
      }
      
      Add(new cEpgMenuTextItem(menuDb->recordingListDb->getStrValue("MD5PATH"), 
                               cString::sprintf("%s%s%s\t%s\t%s/%s", vdr ? vdr : "", vdr ? "\t" : "", 
                                                menuDb->recordingListDb->getStrValue("TITLE"),
                                                menuDb->recordingListDb->getStrValue("SHORTTEXT"),
                                                menuDb->recordingListDb->getStrValue("FOLDER"),
                                                menuDb->recordingListDb->getStrValue("NAME"))));
   }
   
   menuDb->selectRecordingForEvent->freeResult();

   Add(new cOsdItem(tr("All Matches:")));
   cList<cOsdItem>::Last()->SetSelectable(no);

   menuDb->recordingListDb->clear();
   menuDb->recordingListDb->setValue("TITLE", event->Title());

   for (int f = menuDb->selectRecordingForEventByLv->find(); f; f = menuDb->selectRecordingForEventByLv->fetch())
   {
      const char* vdr = 0;
      
#ifdef WITH_PIN     
         if (!cOsd::pinValid && menuDb->recordingListDb->getIntValue("FSK"))
            continue;
#endif

      if (!menuDb->recordingListDb->getValue("OWNER")->isEmpty())
      {
         menuDb->vdrDb->clear();
         menuDb->vdrDb->setValue("UUID", menuDb->recordingListDb->getStrValue("OWNER"));
         
         if (menuDb->vdrDb->find())
            vdr = menuDb->vdrDb->getStrValue("NAME");
      }

      Add(new cEpgMenuTextItem(menuDb->recordingListDb->getStrValue("MD5PATH"), 
                               cString::sprintf("%s%s%s\t%s\t%s/%s", vdr ? vdr : "", vdr ? "\t" : "",
                                                menuDb->recordingListDb->getStrValue("TITLE"),
                                                menuDb->recordingListDb->getStrValue("SHORTTEXT"),
                                                menuDb->recordingListDb->getStrValue("FOLDER"),
                                                menuDb->recordingListDb->getStrValue("NAME"))));
   }
   
   menuDb->selectRecordingForEventByLv->freeResult();

   Display();
}

eOSState cMenuEpgMatchRecordings::ProcessKey(eKeys Key)
{
   eOSState state = cOsdMenu::ProcessKey(Key);
   
   if (state == osUnknown) 
   {
      return osContinue;
   }

   return state;
}

//***************************************************************************
//***************************************************************************
// Class cMenuEpgScheduleItem
//***************************************************************************

class cMenuEpgScheduleItem : public cOsdItem 
{
   public:

      enum eScheduleSortMode { ssmAllThis, ssmThisThis, ssmThisAll, ssmAllAll }; // "which event(s) on which channel(s)"
   
      cMenuEpgScheduleItem(cMenuDb* db, const cEvent* Event, const cChannel* Channel = 0, bool WithDate = no);

      static void SetSortMode(eScheduleSortMode SortMode)  { sortMode = SortMode; }
      static void IncSortMode()                            { sortMode = eScheduleSortMode((sortMode == ssmAllAll) ? ssmAllThis : sortMode + 1); }
      static eScheduleSortMode SortMode()                  { return sortMode; }

      virtual int Compare(const cListObject &ListObject) const;
      virtual bool Update(bool Force = false);
      virtual void SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable);

      const cEvent* event;
      const cChannel* channel;
      bool withDate;
      int timerMatch;
      
   private:

      cMenuDb* menuDb;
      static eScheduleSortMode sortMode;
};

//***************************************************************************
// 
//***************************************************************************

cMenuEpgScheduleItem::eScheduleSortMode cMenuEpgScheduleItem::sortMode = ssmAllThis;

//***************************************************************************
// Object
//***************************************************************************

cMenuEpgScheduleItem::cMenuEpgScheduleItem(cMenuDb* db, const cEvent* Event, const cChannel* Channel, bool WithDate)
{
   menuDb = db;

   event = Event;
   channel = Channel;
   withDate = WithDate;
   timerMatch = tmNone;
   Update(yes);
}

//***************************************************************************
// Compare
//***************************************************************************

int cMenuEpgScheduleItem::Compare(const cListObject &ListObject) const
{
   cMenuEpgScheduleItem *p = (cMenuEpgScheduleItem *)&ListObject;
   int r = -1;

   if (sortMode != ssmAllThis)
      r = strcoll(event->Title(), p->event->Title());

   if (sortMode == ssmAllThis || r == 0)
      r = event->StartTime() - p->event->StartTime();

   return r;
}

//***************************************************************************
// Update
//***************************************************************************

bool cMenuEpgScheduleItem::Update(bool Force)
{
   static const char* TimerMatchChars = " tT";
   static const char* TimerMatchCharsRemote = " sS";

   bool result = false;
   int oldTimerMatch = timerMatch;
   int remote = no;

   if (event)
      menuDb->lookupTimer(event, timerMatch, remote); // -> loads timerDb and vdrDb

   if (Force || timerMatch != oldTimerMatch) 
   {
      if (timerMatch && event)
         tell(0, "Timer match for '%s'", event->Title());

      cString buffer;
      char t = !remote ? TimerMatchChars[timerMatch] : TimerMatchCharsRemote[timerMatch];
      char v = !event ? ' ' : event->Vps() && (event->Vps() - event->StartTime()) ? 'V' : ' ';
      char r = !event ? ' ' : event->SeenWithin(30) && event->IsRunning() ? '*' : ' ';
      const char* csn = channel ? channel->ShortName(true) : 0;
      cString eds = !event ? "" : event->GetDateString();

      if (channel && withDate)
         buffer = cString::sprintf("%d\t%.*s\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 
                                   Utf8SymChars(csn, 999), csn, Utf8SymChars(eds, 6), 
                                   *eds,
                                   !event ? "" : *event->GetTimeString(),
                                   t, v, r,
                                   !event ? "" : event->Title());
      else if (channel)
         buffer = cString::sprintf("%d\t%.*s\t%s\t%c%c%c\t%s", channel->Number(), 
                                   Utf8SymChars(csn, 999), csn,
                                   !event ? "" : *event->GetTimeString(),
                                   t, v, r,
                                   !event ? "" : event->Title());
      else
         buffer = cString::sprintf("%.*s\t%s\t%c%c%c\t%s", Utf8SymChars(eds, 6), *eds,
                                   !event ? "" : *event->GetTimeString(),
                                   t, v, r, !event ? "" :
                                   !event ? "" : event->Title());

      SetText(buffer);
      result = true;
   }

   return result;
}

//***************************************************************************
// SetMenuItem
//***************************************************************************

void cMenuEpgScheduleItem::SetMenuItem(cSkinDisplayMenu* DisplayMenu, 
                                       int Index, bool Current, bool Selectable)
{
   if (!DisplayMenu->SetItemEvent(event, Index, Current, Selectable, channel, 
                                  withDate, (eTimerMatch)timerMatch))
   {
      DisplayMenu->SetItem(Text(), Index, Current, Selectable);
   }
}

//***************************************************************************
//***************************************************************************
// cMenuEpgScheduleSepItem
//***************************************************************************

class cMenuEpgScheduleSepItem : public cOsdItem
{
   public:
      
      cMenuEpgScheduleSepItem(cChannel* Channel, const cEvent* Event);
      ~cMenuEpgScheduleSepItem();

      virtual bool Update(bool Force = false);
      virtual void SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable);

   private:

      const cChannel* channel;
      const cEvent* event;
      cEvent* tmpEvent;
};

cMenuEpgScheduleSepItem::cMenuEpgScheduleSepItem(cChannel* Channel, const cEvent* Event)
{
   channel = Channel;
   event = Event;
   tmpEvent = 0;

   SetSelectable(false);
   Update(true);
}

cMenuEpgScheduleSepItem::~cMenuEpgScheduleSepItem()
{
   delete tmpEvent;
}

bool cMenuEpgScheduleSepItem::Update(bool Force)
{
   if (channel)
   {
      SetText(cString::sprintf("-----\t %s -----", channel ? channel->Name() : *event->GetDateString()));
   }
   else if (event)
   {
      tmpEvent = new cEvent(0);
      tmpEvent->SetTitle(cString::sprintf("-----\t %s -----", *event->GetDateString()));
      SetText(tmpEvent->Title());
   }

   return true;
}

void cMenuEpgScheduleSepItem::SetMenuItem(cSkinDisplayMenu *DisplayMenu, int Index, bool Current, bool Selectable)
{
   if (!DisplayMenu->SetItemEvent(tmpEvent, Index, Current, Selectable, channel, no, tmNone))
      DisplayMenu->SetItem(Text(), Index, Current, Selectable);
}

//***************************************************************************
// cMenuEpgEvent
//***************************************************************************

cMenuEpgEvent::cMenuEpgEvent(const cEvent* Event, bool CanSwitch, bool Buttons)
   : cOsdMenu(tr("Event"))
{
   SetMenuCategory(mcEvent);
   event = Event;

   if (event)
   {
      cChannel* channel = Channels.GetByChannelID(event->ChannelID(), true);

      if (channel)
      {
         SetTitle(channel->Name());
         eTimerMatch TimerMatch = tmNone;
         Timers.GetMatch(event, &TimerMatch);
         
         if (Buttons)
            SetHelp(TimerMatch == tmFull ? tr("Button$Timer") : tr("Button$Record"), 0, 0, CanSwitch ? tr("Button$Switch") : 0);
      }
   }
}

void cMenuEpgEvent::Display()
{
   cOsdMenu::Display();
   DisplayMenu()->SetEvent(event);

#ifdef WITH_GTFT
   cStatus::MsgOsdSetEvent(event);
#endif   

   if (event->Description())
      cStatus::MsgOsdTextItem(event->Description());
}

eOSState cMenuEpgEvent::ProcessKey(eKeys Key)
{
   switch (int(Key))
   {
      case kUp|k_Repeat:
      case kUp:
      case kDown|k_Repeat:
      case kDown:
      case kLeft|k_Repeat:
      case kLeft:
      case kRight|k_Repeat:
      case kRight:
      {
         DisplayMenu()->Scroll(NORMALKEY(Key) == kUp || NORMALKEY(Key) == kLeft, NORMALKEY(Key) == kLeft || NORMALKEY(Key) == kRight);
         cStatus::MsgOsdTextItem(NULL, NORMALKEY(Key) == kUp || NORMALKEY(Key) == kLeft);
         return osContinue;
      }
      
      case kInfo:   return osBack;
      default: break;
   }
   
   eOSState state = cOsdMenu::ProcessKey(Key);
   
   if (state == osUnknown)
   {
      switch (Key)
      {
         case kGreen:
         case kYellow: return osContinue;
         case kOk:     return osBack;
         default: break;
      }
   }

   return state;
}

//***************************************************************************
// cMenuEpgWhatsOn
//***************************************************************************

int cMenuEpgWhatsOn::currentChannel = 0;
const cEvent* cMenuEpgWhatsOn::scheduleEvent = 0;

//***************************************************************************
// Object
//***************************************************************************

cMenuEpgWhatsOn::cMenuEpgWhatsOn(const cEvent* aSearchEvent)
   : cOsdMenu("", CHNUMWIDTH, CHNAMWIDTH, 6, 4)
{
   dispSchedule = no;
   canSwitch = no;
   helpKeys = 0;
   helpKeyTime = 0;
   helpKeyTimeMode = na;
   timerState = 0;
   searchEvent = aSearchEvent;

   menuDb = new cMenuDb;

   cChannel* channel = Channels.GetByNumber(cDevice::CurrentChannel());

   if (channel) 
   {
      currentChannel = channel->Number();
      schedules = cSchedules::Schedules(schedulesLock);
   }

   Timers.Modified(timerState);
   
   menuDb->userTimes->first();     // 'whats on now' should be the first
   
   if (searchEvent)
      LoadQuery(searchEvent);
   else if (menuDb->startWithSched)
      LoadSchedule();
   else
      LoadAt();
}

cMenuEpgWhatsOn::~cMenuEpgWhatsOn()
{
   delete menuDb;
}

//***************************************************************************
// Load At
//***************************************************************************

int cMenuEpgWhatsOn::LoadAt()
{
   dispSchedule = no;
   cUserTimes::UserTime* userTime = menuDb->userTimes->current();
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());

   if (item)
   {
      scheduleEvent = item->event;
      
      if (item->channel)
         currentChannel = item->channel->Number();
   }

   if (userTime->getMode() == cUserTimes::mSearch)
      return LoadSearch(userTime);
   
   Clear();
   Timers.Modified(timerState);
   SetMenuCategory(userTime->getMode() == cUserTimes::mNow ? mcScheduleNow : mcScheduleNext);

   if (isEmpty(userTime->getHHMMStr()))
      SetTitle(cString::sprintf(tr("Overview - %s"), userTime->getTitle()));
   else
      SetTitle(cString::sprintf(tr("Overview - %s (%s)"), userTime->getTitle(), userTime->getHHMMStr()));

   tell(2, "DEBUG: Loading events for '%s' %s", userTime->getTitle(), !isEmpty(userTime->getHHMMStr()) ? userTime->getHHMMStr() : "");

   // events

   for (cChannel* Channel = Channels.First(); Channel; Channel = Channels.Next(Channel)) 
   {
      if (!Channel->GroupSep()) 
      {
         const cSchedule* Schedule = schedules->GetSchedule(Channel);
         const cEvent* Event = 0;

         if (Schedule) 
         {
            switch (userTime->getMode())
            {
               case cUserTimes::mNow:  Event = Schedule->GetPresentEvent();                    break;
               case cUserTimes::mNext: Event = Schedule->GetFollowingEvent();                  break;
               case cUserTimes::mTime: Event = Schedule->GetEventAround(userTime->getTime());  break;
            }
         }
         
         // #TODO
         // hier ein cEpgEvent für das Event aus der Row erzeugen und dieses anstelle von cEvent für das Menü verwenden
         
         if (!Event && !Epg2VdrConfig.showEmptyChannels)
            continue;
         
         Add(new cMenuEpgScheduleItem(menuDb, Event, Channel), Channel->Number() == currentChannel);
      }
      else
      {
         if (strlen(Channel->Name()))
            Add(new cMenuEpgScheduleSepItem(Channel, 0));
      }
   }

   Display();
   SetHelpKeys();

   return done;
}

//***************************************************************************
// Load Search
//***************************************************************************

int cMenuEpgWhatsOn::LoadSearch(const cUserTimes::UserTime* userTime)
{
   cDbStatement* select = 0;

   Clear();
   SetMenuCategory(mcSchedule);
   SetTitle(cString::sprintf(tr("Overview - %s"), userTime->getTitle()));
   tell(2, "DEBUG: Loading events for search '%s'", userTime->getSearch());

   menuDb->searchtimerDb->clear();
   menuDb->searchtimerDb->setValue("NAME", userTime->getSearch());
   
   if (!menuDb->selectSearchTimerByName->find())
      return done;
   
   if (!(select = menuDb->search->prepareSearchStatement(menuDb->searchtimerDb->getRow(), menuDb->useeventsDb)))
      return fail;

#if defined (APIVERSNUM) && (APIVERSNUM >= 20301)
   cChannelsLock channelsLock(false);
   const cChannels* channels = channelsLock.Channels();
#else
   cChannels* channels = &Channels;
#endif

   menuDb->useeventsDb->clear();
   
   for (int res = select->find(); res; res = select->fetch()) 
   {
      menuDb->useeventsDb->find();  // get all fields ..
      
      if (!menuDb->search->matchCriterias(menuDb->searchtimerDb->getRow(), menuDb->useeventsDb->getRow()))
         continue;

      //
      
      const char* strChannelId = menuDb->useeventsDb->getStrValue("CHANNELID");
      const cChannel* channel = channels->GetByChannelID(tChannelID::FromString(strChannelId));
      const cSchedule* schedule = schedules->GetSchedule(channel);
      const cEvent* event = !schedule ? 0 :  schedule->GetEvent(menuDb->useeventsDb->getIntValue("USEID"));

      if (event)
         Add(new cMenuEpgScheduleItem(menuDb, event, channel, yes));
  }

   select->freeResult();
   menuDb->selectSearchTimerByName->freeResult();
   
   Display();
   SetHelpKeys();

   return Count();
}

//***************************************************************************
// Load Schedule
//***************************************************************************

int cMenuEpgWhatsOn::LoadSchedule()
{
   Clear();
   SetCols(7, 6, 4);

   dispSchedule = yes;
   cMenuEpgScheduleItem::SetSortMode(cMenuEpgScheduleItem::ssmAllThis);
   cChannel* channel = Channels.GetByNumber(currentChannel);
   SetMenuCategory(mcSchedule);
   SetTitle(cString::sprintf(tr("Schedule - %s"), channel->Name()));

   if (schedules && channel) 
   {
      const cSchedule* Schedule = schedules->GetSchedule(channel);

      if (Schedule) 
      {
         int lastDay = 0;
         const cEvent* PresentEvent = scheduleEvent ? scheduleEvent : Schedule->GetPresentEvent();
         time_t now = time(0) - Setup.EPGLinger * 60;

         for (const cEvent* ev = Schedule->Events()->First(); ev; ev = Schedule->Events()->Next(ev)) 
         {
            time_t stime = ev->StartTime();
            struct tm tmSTime;
            localtime_r(&stime, &tmSTime);
            
            if (lastDay != 0 && tmSTime.tm_mday != lastDay)
               Add(new cMenuEpgScheduleSepItem(0, ev));

            if (ev->EndTime() > now || ev == PresentEvent)
               Add(new cMenuEpgScheduleItem(menuDb, ev, channel), ev == PresentEvent);

            lastDay = tmSTime.tm_mday;
         }
      }
   }

   Display();
   SetHelpKeys();

   return done;
}

//***************************************************************************
// Load Query
//***************************************************************************

int cMenuEpgWhatsOn::LoadQuery(const cEvent* searchEvent)
{
   Clear();
   SetMenuCategory(mcSchedule);
   cMenuEpgScheduleItem::SetSortMode(cMenuEpgScheduleItem::ssmAllThis);

   for (cChannel* channel = Channels.First(); channel; channel = Channels.Next(channel)) 
   {      
      const cSchedule* schedule = schedules->GetSchedule(channel);
      
      if (schedule) 
      {
         for (const cEvent* event = schedule->Events()->First(); event; event = schedule->Events()->Next(event))
         {
            if (rep(event->Title(), searchEvent->Title()) == success &&
                event->StartTime() > time(0))
               Add(new cMenuEpgScheduleItem(menuDb, event, channel, yes));
         }
      }
   }

   Sort();
   SetTitle(cString::sprintf("%d %s - %s", Count(), tr("Search results"), searchEvent->Title()));

   Display();
   SetHelpKeys();

   return Count();
}

//***************************************************************************
// Display
//***************************************************************************

void cMenuEpgWhatsOn::Display()
{
   cOsdMenu::Display();

#ifdef WITH_GTFT
   if (Count() > 0) 
   {
      int ni = 0;

      for (cOsdItem *item = First(); item; item = Next(item)) 
         cStatus::MsgOsdEventItem(((cMenuEpgScheduleItem*)item)->event, item->Text(), ni++, Count());
   }
#endif
}

//***************************************************************************
// Update
//***************************************************************************

bool cMenuEpgWhatsOn::Update()
{
   bool result = false;

   for (cOsdItem* item = First(); item; item = Next(item)) 
   {
      if (((cMenuEpgScheduleItem*)item)->Update())
         result = true;
   }

   return result;
}

//***************************************************************************
// SetHelpKeys
//***************************************************************************

void cMenuEpgWhatsOn::SetHelpKeys()
{
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem *)Get(Current());
   canSwitch = no;
   int NewHelpKeys = 0;

   if (item) 
   {
      if (item->timerMatch == tmFull)
         NewHelpKeys |= 0x02;    // "Timer"
      else
         NewHelpKeys |= 0x01;    // "Record"

      if (!dispSchedule)
         NewHelpKeys |= 0x04;    // "Schedule"
      
      if (item->channel)
      {
         if (item->channel->Number() != cDevice::CurrentChannel()) 
         {
            NewHelpKeys |= 0x10; // "Switch"
            canSwitch = yes;
         }
      }
   }

   cUserTimes::UserTime* userTimeGreen = !dispSchedule ? menuDb->userTimes->getNext() : menuDb->userTimes->current();
   
   if (NewHelpKeys != helpKeys || helpKeyTime != userTimeGreen->getTime() || helpKeyTimeMode != userTimeGreen->getMode())
   {
      const char* Red[] = { 0, tr("Button$Record"), tr("Button$Timer") };

      if (searchEvent)
      {
         helpKeys = 0;
         helpKeyTime = 0;
         helpKeyTimeMode = na;
         
         SetHelp(Red[NewHelpKeys & 0x03], 0, 0, 0);
      }
      else
      {
         SetHelp(Red[NewHelpKeys & 0x03],                                   // red
                 userTimeGreen->getHelpKey(),                               // green
                 !dispSchedule
                 ? tr("Button$Schedule")                                    // yellow
                 : menuDb->userTimes->current() == menuDb->userTimes->getFirst()
                 ? tr(menuDb->userTimes->getNext()->getHelpKey())           // yellow
                 : tr(menuDb->userTimes->getFirst()->getHelpKey()),         // yellow
                 Epg2VdrConfig.xchgOkBlue ? tr("Button$Info") : canSwitch ? tr("Button$Switch") : 0);  // blue

         helpKeyTime = userTimeGreen->getTime();
         helpKeyTimeMode = userTimeGreen->getMode();
         helpKeys = NewHelpKeys;
      }
   }
}

//***************************************************************************
// Switch
//***************************************************************************

eOSState cMenuEpgWhatsOn::Switch()
{
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
      
   if (cDevice::PrimaryDevice()->SwitchChannel(item->channel, true))
      return osEnd;

   Skins.Message(mtError, tr("Can't switch channel!"));

   return osContinue;
}

//***************************************************************************
// Record
//***************************************************************************

eOSState cMenuEpgWhatsOn::Record()
{
   int timerMatch = tmNone;
   cEpgTimer* timer = 0;
   long timerid = 0;
   int remote = no;
   cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());

   if (!item)
      return osContinue;

   if (timerid = menuDb->lookupTimer(item->event, timerMatch, remote)) // -> loads timerDb and vdrDb
   {
      menuDb->timerDb->clear();
      menuDb->timerDb->setValue("ID", timerid);

      if (!menuDb->selectTimerById->find())
      {
         tell(0, "Fatal, Can't find timer (%ld)", timerid);
         return osContinue;
      }

      timer = newTimerObjectFromRow(menuDb->timerDb->getRow(), menuDb->vdrDb->getRow());
   }
   
   if (timer)
      return AddSubMenu(new cMenuEpgEditTimer(menuDb, timer));
   
   // neuen Timer anlegen

   cDbRow* timerRow = newRowFromEvent(item->event);
   char timerDefaultVDRuuid[150+TB]; *timerDefaultVDRuuid = 0;

   menuDb->getParameter(menuDb->user.c_str(), "timerDefaultVDRuuid", timerDefaultVDRuuid);

   // Menü bei 'aktuellem' Event Timer Dialog öffen -> #TODO

   if (timer && timer->Event() && timer->Event()->StartTime() < time(0) + NEWTIMERLIMIT)
   {
      // timer = newTimerObjectFromRow(timerRow, xxxxx);
      // return AddSubMenu(new cMenuEpgEditTimer(menuDb, timer));
   }

   // ansonsten direkt anlegen

   menuDb->createTimer(timerRow, isEmpty(timerDefaultVDRuuid) || Epg2VdrConfig.createTimerLocal ? Epg2VdrConfig.uuid : timerDefaultVDRuuid);
   delete timerRow;

   if (HasSubMenu())
      CloseSubMenu();

   sleep(1);
   
   if (Update())
      Display();
   
   SetHelpKeys();
   
   return osContinue;
}

//***************************************************************************
// ProcessKey
//***************************************************************************

eOSState cMenuEpgWhatsOn::ProcessKey(eKeys Key)
{
   bool HadSubMenu = HasSubMenu();
   eOSState state = cOsdMenu::ProcessKey(Key);
   
   if (state == osUnknown) 
   {
      if (searchEvent && Key != kOk && Key != kRed && Key != kRecord)
         return osContinue;

      if (Epg2VdrConfig.xchgOkBlue && !HasSubMenu())
      {
         if      (Key == kBlue) Key = kOk;
         else if (Key == kOk)   Key = kBlue;
      }
      
      switch (Key) 
      {
         case k1:       // search repeat of event
         {
            cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
            
            if (item) 
               return AddSubMenu(new cMenuEpgWhatsOn(item->event));

            break;
         }

         case k2:      // search matching recordings
         {
            cMenuEpgScheduleItem* item = (cMenuEpgScheduleItem*)Get(Current());
            
            if (item)
               return AddSubMenu(new cMenuEpgMatchRecordings(menuDb, item->event));

            break;
         }

         case k3:  // search timer dialog
         {
            return AddSubMenu(new cMenuEpgSearchTimers());
         }

         case kRecord:
         case kRed:
            return Record();

         case kYellow: 
         {
            if (!dispSchedule)
            {
               cMenuEpgScheduleItem* mi = (cMenuEpgScheduleItem*)Get(Current());
               
               if (mi) 
               {
                  scheduleEvent = mi->event;
                  currentChannel = mi->channel->Number();
                  LoadSchedule();
               }
            }
            else
            {
               if (menuDb->userTimes->current() == menuDb->userTimes->getFirst())
                  menuDb->userTimes->next();
               else
                  menuDb->userTimes->first();
               
               LoadAt();
            }
            
            return osContinue;
         }
         case kGreen:  
         {
            if (!dispSchedule)
               menuDb->userTimes->next();
            
            LoadAt();

            return osContinue;
         }

         case kBlue:   
         {
            if (canSwitch)
               return Switch();

            break;
         }

         case kInfo:
         case kOk:
         {
            const cEvent* event = ((cMenuEpgScheduleItem *)Get(Current()))->event;

            if (Count() && event)
               return AddSubMenu(new cMenuEpgEvent(event, canSwitch, true));

            break;
         }

         default: 
            break;
      }
   }

    if (!HasSubMenu())
   {
      if (HadSubMenu && Update())
         Display();

      if (Key != kNone)
         SetHelpKeys();
   }

   return state;
}

//***************************************************************************

#endif // defined (APIVERSNUM) && (APIVERSNUM >= 20301)
